@@ -19,26 +19,53 @@
 
 
 .ANNsplitRule <- c("STD", "MIDPT", "FAIR", "SL_MIDPT", "SL_FAIR", "SUGGEST")
+.matrixlike <- function(x) return(!is.null(dim(x)))
+
 
 kNN <- function(x, k, sort = TRUE, search = "kdtree", bucketSize = 10,
   splitRule = "suggest", approx = 0) {
 
+  search <- pmatch(toupper(search), c("KDTREE", "LINEAR", "DIST"))
+  if(is.na(search)) stop("Unknown NN search type!")
+
+  k <- as.integer(k)
+  if(k < 1) stop("Illegal k: needs to be k>=1!")
+
+  ### dist search
+  if(search == 3) {
+    if(!is(x, "dist"))
+      if(.matrixlike(x)) x <- dist(x)
+      else stop("x needs to be a matrix to calculate distances")
+  }
+
+  ### get kNN from a dist object
+  if(is(x, "dist")) {
+    x <- as.matrix(x)
+    diag(x) <- NA
+
+    if(k >= nrow(x)) stop("Not enought neighbors in data set!")
+
+    o <- t(apply(x, 1, order, decreasing = FALSE))
+    id <- o[,1:k]
+    dimnames(id) <- list(rownames(x), 1:k)
+    d <- t(sapply(1:nrow(id), FUN = function(i) x[i, id[i,]]))
+    dimnames(d) <- list(rownames(x), 1:k)
+
+    return(list(dist = d, id = id, k = k))
+  }
+
   ## make sure x is numeric
+  if(!.matrixlike(x)) stop("x needs to be a matrix to calculate distances")
   x <- as.matrix(x)
   if(storage.mode(x) == "integer") storage.mode(x) <- "double"
   if(storage.mode(x) != "double") stop("x has to be a numeric matrix.")
 
-
-  k <- as.integer(k)
-
-  if(k < 1) stop("Illegal k: needs to be k>=1!")
   if(k >= nrow(x)) stop("Not enought neighbors in data set!")
 
+
   splitRule <- pmatch(toupper(splitRule), .ANNsplitRule)-1L
   if(is.na(splitRule)) stop("Unknown splitRule!")
 
-  search <- pmatch(toupper(search), c("KDTREE", "LINEAR"))
-  if(is.na(search)) stop("Unknown NN search type!")
 
   ## returns NO self matches
   ret <- kNN_int(as.matrix(x), as.integer(k),
