@@ -20,34 +20,58 @@
 optics <- function(x, eps, minPts = 5, eps_cl, search = "kdtree",
   bucketSize = 10, splitRule = "suggest", approx = 0) {
 
-  ## make sure x is numeric
-  x <- as.matrix(x)
-  if(storage.mode(x) == "integer") storage.mode(x) <- "double"
-  if(storage.mode(x) != "double") stop("x has to be a numeric matrix.")
-
   splitRule <- pmatch(toupper(splitRule), .ANNsplitRule)-1L
   if(is.na(splitRule)) stop("Unknown splitRule!")
 
-  search <- pmatch(toupper(search), c("KDTREE", "LINEAR"))
+  search <- pmatch(toupper(search), c("KDTREE", "LINEAR", "DIST"))
   if(is.na(search)) stop("Unknown NN search type!")
 
+  ### dist search
+  if(search == 3) {
+    if(!is(x, "dist"))
+      if(.matrixlike(x)) x <- dist(x)
+      else stop("x needs to be a matrix to calculate distances")
+  }
+
+  ## for dist we provide the R code with a frNN list and no x
+  frNN <- list()
+  if(is(x, "dist")) {
+    frNN <- frNN(x, eps)
+    ## add self match and use C numbering
+    frNN$id <- lapply(1:length(frNN$id),
+      FUN = function(i) c(i-1L, frNN$id[[i]]-1L))
+    frNN$dist <- lapply(1:length(frNN$dist),
+      FUN = function(i) c(0, frNN$dist[[i]])^2)
+
+    x <- matrix()
+    storage.mode(x) <- "double"
+
+  }else{
+    if(!.matrixlike(x)) stop("x needs to be a matrix")
+    ## make sure x is numeric
+    x <- as.matrix(x)
+    if(storage.mode(x) == "integer") storage.mode(x) <- "double"
+    if(storage.mode(x) != "double") stop("x has to be a numeric matrix.")
+  }
+
   ret <- optics_int(as.matrix(x), as.double(eps), as.integer(minPts),
     as.integer(search), as.integer(bucketSize),
-    as.integer(splitRule), as.double(approx))
+    as.integer(splitRule), as.double(approx), frNN)
+
+  ret$minPts <- minPts
+  ret$eps <- eps
+  ret$eps_cl <- NA
 
   ### find clusters
   if(!missing(eps_cl)) ret <- optics_cut(ret, eps_cl)
 
-  ret$eps <- eps
-  ret$minPts <- minPts
-
   class(ret) <- "optics"
   ret
 }
 
 
 ### extract clusters
-optics_cut <- function(x, eps) {
+optics_cut <- function(x, eps_cl) {
   reachdist <- x$reachdist[x$order]
   coredist <- x$coredist[x$order]
   n <- length(x$order)
@@ -55,8 +79,8 @@ optics_cut <- function(x, eps) {
 
   clusterid <- 0L         ### 0 is noise
   for(i in 1:n) {
-    if(reachdist[i] > eps) {
-      if(coredist[i] <= eps) {
+    if(reachdist[i] > eps_cl) {
+      if(coredist[i] <= eps_cl) {
         clusterid <- clusterid + 1L
         cluster[i] <- clusterid
       }else{
@@ -67,7 +91,7 @@ optics_cut <- function(x, eps) {
     }
   }
 
-  x$eps_cl <- eps
+  x$eps_cl <- eps_cl
   ### fix the order so cluster is in the same order as the rows in x
   cluster[x$order] <- cluster
   x$cluster <- cluster
@@ -77,7 +101,10 @@ optics_cut <- function(x, eps) {
 
 print.optics <- function(x, ...) {
   cat("OPTICS clustering for ", length(x$order), " objects.", "\n", sep = "")
-  cat("Parameters: eps = ", x$eps, ", minPts = ", x$minPts, "\n", sep = "")
+  cat("Parameters: ", "minPts = ", x$minPts,
+    ", eps = ", x$eps,
+    ", eps_cl = ", x$eps_cl,
+    "\n", sep = "")
   if(!is.null(x$cluster)) {
     cl <- unique(x$cluster)
     cl <- length(cl[cl!=0L])
