@@ -1,4 +1,3 @@
-
 library("dbscan")
 library("testthat")
 
@@ -14,9 +13,11 @@ x <- cbind(
 
 ### calculate LOF score
 system.time(lof <- lof(x, k=4))
-
 expect_identical(length(lof), nrow(x))
 
+system.time(lof_d <- lof(dist(x), k=4))
+expect_equal(lof, lof_d)
+
 ## compare with lofactor from DMwR
 if(requireNamespace("DMwR", quietly = TRUE)) {
   system.time(lof_DMwr <- DMwR::lofactor(x, k=4))
